<%@page buffer="none" session="false" taglibs="c,cms,fmt,fn" trimDirectiveWhitespaces="true" import="org.opencms.main.*, org.opencms.search.*, org.opencms.file.*, org.opencms.file.types.*, org.opencms.jsp.*,org.opencms.jsp.util.*, org.opencms.util.*, java.util.*,java.io.*" %><%!
private final static org.apache.log4j.Logger log=org.apache.log4j.Logger.getLogger("it.teamquality.jsp.GlideMethods");
private final static int OUTPUT_MAX_LENGTH=1500;
private final static String GLIDE_MODULE_NAME="/system/modules/it.teamquality.glide";
private final static String PARAM_GRUNT_NAME="grunt";
private final static String PARAM_GRUNT_START="start";
private final static String PARAM_GRUNT_STOP="stop";
private final static String KEY_OUTPUT="output";
private final static String KEY_PROCESS="process";
private final static String LINE_BREAK="<br>";
private final static String COOKIE_LIVE_RELOAD="liveReload";
private final static String COOKIE_AUTO_PUBLISH="autoPublish";
private final static String COOKIE_STAY_OPEN="stayOpen";
private final static String COOKIE_HORIZONTAL_LAYOUT="horizontalLayout";

boolean isGlideUser(HttpServletRequest request, Map<String,CmsJspContentAccessValueWrapper> valueMap) throws org.opencms.main.CmsException {
	String userGroup=valueMap.get("UserGroup").toString();
	org.opencms.file.CmsObject cmsObj=org.opencms.flex.CmsFlexController.getCmsObject(request);
	return cmsObj.userInGroup(cmsObj.getRequestContext().getCurrentUser().getName(), userGroup);
}

/**
 * Cuts the output to a maximum of OUTPUT_MAX_LENGTH characters.
 */
String outputString(StringBuffer output) {
	String cut;
	if (output.length()>OUTPUT_MAX_LENGTH) {
		cut=output.substring(output.length()-OUTPUT_MAX_LENGTH);
		cut=cut.substring(cut.indexOf(LINE_BREAK)+LINE_BREAK.length(),cut.length());
	} else {
		cut=output.toString();
	}
	//log.debug("output length = "+output.length());
	return cut;
}

String outputString(HttpServletRequest request) {
	ServletContext context = request.getSession().getServletContext();
	Object output=context.getAttribute(KEY_OUTPUT);
	if (output instanceof StringBuffer) return outputString((StringBuffer) output); // instanceof also checks for null
	else return "";
}

boolean isGruntRunning(HttpServletRequest request) {
	return request.getSession().getServletContext().getAttribute(KEY_PROCESS) instanceof Process;
}

void gruntStop(HttpServletRequest request) {
	if (!isGruntRunning(request)) return;
	ServletContext context = request.getSession().getServletContext();
	log.debug(KEY_PROCESS+"="+context.getAttribute(KEY_PROCESS));
	Process process=(Process) context.getAttribute(KEY_PROCESS);
	process.destroy();
	try {
		process.waitFor();
	} catch (InterruptedException e) {}
	if (context.getAttribute(KEY_OUTPUT) instanceof StringBuffer) ((StringBuffer) context.getAttribute(KEY_OUTPUT)).append("Destroyed process. "+new Date()+LINE_BREAK);
	context.setAttribute(KEY_PROCESS, null);
}

void gruntStart(HttpServletRequest request) {
	String moduleName=request.getParameter("moduleName");
	String outputStyle=request.getParameter("outputStyle");
	String environment=request.getParameter("environment");
	String mountPath=request.getParameter("mountPath");
	String reloadPort=request.getParameter("reloadPort");
	gruntStart(request, moduleName, outputStyle, environment, mountPath, reloadPort);
}

void gruntStart(HttpServletRequest request, Map<String,CmsJspContentAccessValueWrapper> valueMap) {
	String moduleName=valueMap.get("ModuleName").toString();
	String outputStyle=valueMap.get("OutputStyle").toString();
	String environment=valueMap.get("Environment").toString();
	String mountPath=valueMap.get("MountPath").toString();
	String reloadPort=valueMap.get("ReloadPort").toString();
	gruntStart(request, moduleName, outputStyle, environment, mountPath, reloadPort);
}

void gruntStart(HttpServletRequest request, String moduleName, String outputStyle, String environment, String mountPath, String reloadPort) {
	if (isGruntRunning(request)) gruntStop(request); // ensure that it's not already running

	/* prepare final parameters */
	final String moduleNameF=moduleName;
	final String outputStyleF=outputStyle;
	final String environmentF=environment;
	final String mountPathF=mountPath;
	final String reloadPortF=reloadPort;
	final ServletContext contextF = request.getSession().getServletContext();

	Runnable runner = new Runnable() {
		public void run() {
			StringBuffer output=new StringBuffer();
			contextF.setAttribute(KEY_OUTPUT, output);

			try {
				ProcessBuilder pb = new ProcessBuilder("node", "node_modules/grunt-cli/bin/grunt","--no-color","--moduleName="+mountPathF+moduleNameF,"--outputStyle="+outputStyleF,"--environment="+environmentF,"--reloadPort="+reloadPortF/*,"-d","-v"*/); // ToDo: add a "verbose debug" parameter
				pb.directory(new File(mountPathF+GLIDE_MODULE_NAME));
				pb.redirectErrorStream(true);
				output.append("Starting process. "+new Date()+LINE_BREAK);
				Process process=pb.start();
				contextF.setAttribute(KEY_PROCESS, process);
				BufferedReader outputBR=new BufferedReader(new InputStreamReader(process.getInputStream(), "UTF-8"));
				String line;
				while ((line=outputBR.readLine())!=null) output.append(line+LINE_BREAK);
			} catch (Throwable th) {
				log.error("error starting grunt", th);
				output.append("error starting grunt: "+th+LINE_BREAK);
			}
		}
	};
	Thread thread=new Thread(runner);
	thread.start();
	try {
		Thread.sleep(1000); // give grunt a little time to start (optional)
	} catch (InterruptedException e) {}
}

void publish(String moduleName, javax.servlet.jsp.PageContext pageContext, javax.servlet.http.HttpServletRequest request, javax.servlet.http.HttpServletResponse response) throws Exception {
	log.debug("autopublishing "+moduleName);
	CmsJspActionElement cmsJAE = new CmsJspActionElement(pageContext, request, response);
	CmsObject cmsObj = cmsJAE.getCmsObject();
	if (cmsObj.getLock(moduleName).isUnlocked()) cmsObj.lockResource(moduleName);
	else cmsObj.changeLock(moduleName);
	OpenCms.getPublishManager().publishResource(cmsObj, moduleName);
	OpenCms.getPublishManager().waitWhileRunning();
	cmsObj.unlockResource(moduleName);
	log.debug("autopublished "+moduleName);
}

void autoPublish(String moduleName, javax.servlet.jsp.PageContext pageContext, javax.servlet.http.HttpServletRequest request, javax.servlet.http.HttpServletResponse response) {
	ServletContext context = request.getSession().getServletContext();
	CmsJspActionElement cmsJAE = new CmsJspActionElement(pageContext, request, response);
	String cookieValue=CmsRequestUtil.getCookieValue(cmsJAE, COOKIE_AUTO_PUBLISH);
	try {
		if(isGruntRunning(request) && cookieValue!=null && Boolean.parseBoolean(cookieValue)) {
			publish(moduleName, pageContext, request, response);
			if (context.getAttribute(KEY_OUTPUT) instanceof StringBuffer) ((StringBuffer) context.getAttribute(KEY_OUTPUT)).append("Published "+moduleName+LINE_BREAK);
		}
	} catch (Throwable th){
		log.warn("Error while publishing "+moduleName, th);
		if (context.getAttribute(KEY_OUTPUT) instanceof StringBuffer) ((StringBuffer) context.getAttribute(KEY_OUTPUT)).append("Error while publishing "+moduleName+": "+th+LINE_BREAK);
	}
}
%>